
split_pipes = (f) ->
    (s) ->
        # s without surrounding pipes
        if s is "|"
                sp = ""
        else if s.substring(0,1) isnt "|"
                sp = s
        else
                sp = s.substring(1, s.length - 1)

        # The array of values delimited by pipes
        a = sp.split("|")

        # Return them comma separated, or if the array contains no
        # characters, just return a space char
        (if a.join("") then _.map(a,f).join(", ") else "&nbsp;")

lemma = (s) ->
    if $.bbq.getState("language") == "sv-1800"
        if s.startsWith("dalinm--")
            prefix = "dalin"
            s = s.replace("dalinm--", "")
        else if s.startsWith("swedbergm--")
            prefix = "swedberg"
            s = s.replace("swedbergm--", "")
        else
            prefix = "saldo"
        """#{s}<span id="name_space">#{prefix}</span>"""
    else
        s

saldo_link = (s) ->
    if ":" in s
        s = s.split(":")[0]
    """<a target="_blank" href="#{config.karp_address}&searchTab=extended&extended=and%7Csense%7Cequals%7C#{s}&search=extended%7C%7Cand%7Csense%7Cequals%7C#{s}">#{s}</a>"""

sense_link = (s) ->
    sense = s.split(":")[0]
    prob = s.split(":")[-1..][0]
    if prob is "-1.000"
        """<a target="_blank" href="#{config.karp_address}&searchTab=extended&extended=and%7Csense%7Cequals%7C#{sense}&search=extended%7C%7Cand%7Csense%7Cequals%7C#{sense}">#{sense}</a>"""
    else
        """<a target="_blank" href="#{config.karp_address}&searchTab=extended&extended=and%7Csense%7Cequals%7C#{sense}&search=extended%7C%7Cand%7Csense%7Cequals%7C#{sense}">#{sense}</a> (#{prob})"""

lemgram_link = (s, show_prob = false, prob = 0) ->
    if ":" in s
        s = s.split(":")[0]
    if $.bbq.getState("language") == "sv-1800"
        if s.startsWith("dalinm--")
            prefix = "dalin"
            text = s.replace("dalinm--", "")
        else if s.startsWith("swedbergm--")
            prefix = "swedberg"
            text = s.replace("swedbergm--", "")
        else
            prefix = "saldo"
            text = s
        """<a target="_blank" href="#{config.karp_address}&searchTab=extended&extended=and%7Clemgram%7Cequals%7C#{s}&search=extended%7C%7Cand%7Clemgram%7Cequals%7C#{s}">#{text}<span id="name_space">#{prefix}</span></a>"""
    else
        if show_prob isnt false
            """<a target="_blank" title="#{prob}" href="#{config.karp_address}&searchTab=extended&extended=and%7Clemgram%7Cequals%7C#{s}&search=extended%7C%7Cand%7Clemgram%7Cequals%7C#{s}">#{s}</a>"""
        else
            """<a target="_blank" href="#{config.karp_address}&searchTab=extended&extended=and%7Clemgram%7Cequals%7C#{s}&search=extended%7C%7Cand%7Clemgram%7Cequals%7C#{s}">#{s}</a>"""

complemgram_link = (s) ->
    complems = []
    for comp in s.split("|") when comp isnt ""
        comps = []
        prob = comp.split("+")[-1..][0].split(":")[-1..][0]
        for lem in comp.split("+")
            # if lem is comp.split("+")[-1..][0]
            # prob = prob.split("e")[0] + "×10 <sup>" + prob.split("e")[1] + "</sup>"
            lem = lem.split(":")[0]
            comps.push lemgram_link lem, true, prob
        if comps.length > 0
            complems.push comps.join "+"
    complems.join ", "

compwf = (s) ->
    compwf = []
    for comp in s.split("|")
        if comp isnt ""
            compwf.push comp
    compwf.join ",<br>"

xml_attr_value = (x,a) ->
    if x.attributes.getNamedItem(a) isnt null
      x.attributes.getNamedItem(a).value

delay_viewport_change = () -> window.setTimeout($.fn.handleViewPortChange, 100)

# How to present the different columns (link to Karp etc)
display_column = do ->
    lookup =
        msd:    (s) -> s.split(".").join ". "
        lemma:      split_pipes lemma
        lex:        split_pipes ((x) -> lemgram_link(x))  # Fix to preserve default arg
        sense:      split_pipes sense_link
        saldo:      split_pipes saldo_link
        prefix:     split_pipes ((x) -> lemgram_link(x))
        suffix:     split_pipes ((x) -> lemgram_link(x))
        complemgram: complemgram_link
        compwf:     compwf
        blingbring: split_pipes _.identity
        swefn: split_pipes _.identity
        sentiment: split_pipes _.identity
        # lemprob:    split_pipes _.identity
    (attr) -> lookup[attr] or (x) -> if x == "" then '&nbsp;' else x


# flatten sentences with NE-structures
flatten_ne_xml = (sent) ->
    flat_sent = (new XMLSerializer()).serializeToString sent
    flat_sent = jQuery.parseXML(flat_sent).firstChild
    for ne in $(flat_sent).find("ne")
        for w in $(ne).find("w")
            flat_sent.insertBefore(w, ne)
        flat_sent.removeChild(ne)
    flat_sent


# Makes a table and deptree for a sentence
# First argument is a tuple of settings, second argument is the sentence in XML
#
# tabulate_sentence :: ([Column], Bool) -> XML -> IO ()
tabulate_sentence = (attributes, make_deptrees, sent, index, div, info_div) ->
    table = $("""<table id="#{sent.nodeName}-#{index}" class="table table-striped table-bordered table-condensed"/>""")
    if sent.nodeName != "sentence"
        # no margin at the end of a table
        table.css("margin-bottom","0px");

    fill_table = ->

        prepend_to_table = (div) ->
            td = $("<td/>")
                    .attr("colspan", attributes.length + 1)  # word is not in attributes list
                    .css("background-color", "#FFFFFF")
                    .append(div)

            $("<tr/>")
                .append(td)
                .prependTo table

        # append row for one word
        append_row_to_table = (row) ->
            table.append $("<tr/>").append (for col in row
                    $("<td>").append(col)
                )...

        tabulate_word = (word, index) ->
            [$("<span/>").text word.textContent].concat _.map attributes,
                (attr) ->
                    value = xml_attr_value word, attr
                    if typeof value == "undefined"
                        value = ""
                    span = $("""<span id="#{attr}-#{index}"/>""").html (display_column attr) value
                    if _.contains(["sv", "sv-dev", "sv-1800"], $.bbq.getState("language"))
                        if _.contains(["pos", "deprel", "msd", "sentiment"], attr)
                            loc = localization_info attr, value
                            # span.hover -> info_div.localize_element loc
                            span.localize_element loc, true
                    else # For other modes only localize pos
                        if _.contains(["pos"], attr)
                            loc = localization_info attr, value
                            span.localize_element loc, true
                    span

        # Make header
        if sent.nodeName == "sentence"
            token = $("<th>").text "token"
            token.localize_element (localization_info "annotation", "token"), true
            header = [token]
            # header = [$("<th>").localize_element { sv: "ord", en: "word" }]
            for attr in attributes
                span = $("<th>#{attr}</th>")
                loc = localization_info "annotation", attr
                span.localize_element loc, true
                header.push span
            # Add header to table
            table.append $("<tr/>").append header

            # Define the info div on the top
            if _.contains(["sv", "sv-dev", "sv-1800"], $.bbq.getState("language"))
                info_div = $('<div style="padding-left: 24px"/>')

        # Write information about the words and their attributes
        index = 0
        for child in $(sent).children()
            index += 1
            if child.nodeName == "w"
                append_row_to_table (tabulate_word child, index)

            else
                # Keep on parsing until word level is reached
                td = $("<td/>")
                        .attr("colspan", attributes.length + 1)
                table.append $("<tr/>").append(td)
                display attributes, make_deptrees, child, td, info_div


        # Fix dependency trees
        if sent.nodeName == "sentence"
            flat_sent = flatten_ne_xml sent

            if make_deptrees
                iframe = $('<iframe src="deptrees/index.html">')
                prepend_to_table iframe

                iframe.load ->
                    try
                        i_window = iframe.get(0).contentWindow
                        json_sent = i_window.sentence_xml_to_json flat_sent
                        # console.log json_sent
                        i_window.draw_deptree.call i_window, json_sent, (msg) ->
                            [[k,v]] = _.pairs msg
                            info_div.localize_element localization_info k,v

                    catch e
                        console.log e

                    delay_viewport_change()

        if sent.nodeName == "sentence"
            prepend_to_table info_div

        delay_viewport_change()

    table.append $("<div style='height: 30px'/>").one 'inview', ->
        # console.log "Showing more from sentence #{xml_attr_value sent, 'id'}", table
        $(this).detach()
        fill_table()
        false


# First argument is the sentence_handler (tabulate_sentence partially applied with settings),
# second argument is the current position in the XML, and the div to append to.
#
# display :: (XML -> IO ()) -> (XML, DOM) -> IO ()
display = (attributes, make_deptrees, tag, div, info_div) ->
    disabled = $("#show_tags").attr("checked") isnt "checked"

    index = 1

    walk_tree = (tag, div) ->
        if tag and tag.nodeName == "ne"
            parents = [tag]
        else
            parents = $(tag).children()

        for child in parents
            header = $ "<span class='tag_header'>#{child.nodeName}</span>"
            for attr in child.attributes
                span = $ """<span class="name">#{attr.name}</span><span class="value">#{attr.value}</span>"""
                # Add mouse-over info for named entities
                if attr.nodeName in ["ex", "type", "subtype"]
                    loc = localization_info "ne", attr.value
                    span.localize_element loc, true
                if attr.nodeName in ["lix", "ovix", "nk"]
                    loc = localization_info "misc", attr.nodeName
                    span.localize_element loc, true
                header.append span

            xml_button = $("<span class='tag_xmlbutton'>XML</span>").localize_element(
                    en: " [Show XML]"
                    sv: " [Visa XML]"
                ).click ->
                    new_window (new XMLSerializer()).serializeToString child
                    false

            closed = header.clone().removeClass("tag_header").addClass("tag_closed").addClass("hide")
            footer = $ """
                <span class='tag_footer'>#{child.nodeName}</span>
            """
            contents = $ "<div width='100%'/>"
            outline = $("<div width='100%' class='tag_outline table-bordered'/>")
                .append closed, header, xml_button, contents, footer
            fields = [header,footer,xml_button]

            if disabled
                e.addClass("disabled") for e in [outline].concat(fields)

            for el in [header,footer]
                el.click ->
                    closed.removeClass("hide")
                    e.addClass("hide") for e in [contents].concat(fields)
                    delay_viewport_change()

            closed.click ->
                closed.addClass("hide")
                e.removeClass("hide") for e in [contents].concat(fields)
                delay_viewport_change()

            div.append outline

            if child.nodeName == "sentence" or child.nodeName == "ne"
                outline.addClass "tag_sentence"
                contents.append tabulate_sentence attributes, make_deptrees, child, index, div, info_div
                index += 1

            else
                walk_tree child, contents

    walk_tree tag, div


$(document).ready ->
    $("#show_tags").change ->
        disabled = $(this).attr("checked") is "checked"
        query = $(".tag_header,.tag_xmlbutton,.tag_outline,.tag_footer,.tag_floatfix")
        if disabled
            query.removeClass("disabled")
        else
            query.addClass("disabled")
    return

new_window = (content) ->
    w = window.open '', '_blank'
    w.document.open "text/plain", "replace"
    # This will make a body tag in the document
    w.document.write "hello"
    # Replace the body with the content
    $("body", w.document).empty().append($("<pre/>").text(content))
    # Without this, Firefox thinks the page is already loaded
    w.document.close()

hide_column = (attribute) ->
    # Hide headers
    headers = $("#result").find($("th").filter(() -> return $(this).text() == attribute))
    for header in headers
        $(header).attr("style", "display:none;")
    index = _.indexOf($("#result").find("th"), headers[0])
    # Hide all tr elements belonging to this column
    for row in $("#result").find("tr")
        if $(row).children("td").length == $(headers[0]).parent().children("th").length
            $($(row).children("td")[index]).attr("style", "display:none;")

window.make_table = (data) ->
    attributes = []

    words = data.getElementsByTagName("w")
    if words.length > 0
        for attr in words[0].attributes
            attributes.push attr.name

    # Only make dependency trees if all required attributes are present
    make_deptrees = _.all ["pos", "ref", "deprel"], (attr) -> _.contains(attributes, attr)

    # Then remove pos if msd is present, remove ref and dephead
    if _.contains(["sv", "sv-dev", "sv-1800"], $.bbq.getState("language"))
        attributes = _.without attributes, "ref"
        attributes = _.without attributes, "dephead"
        if _.contains(attributes, "msd")
            attributes = _.without attributes, "pos"

    do ->
        $("#result").empty().append tables_div = $ "<div/>"
        corpus = (data.getElementsByTagName "corpus")[0]
        display attributes, make_deptrees, corpus, tables_div, null

    link = $(data.getElementsByTagName("corpus")).attr("link")

    # Add XML download button if there is a download link
    if(typeof link != "undefined")
        xml_button = """<a href="#{link}" download="corpus.xml" id="extra_buttons" class="btn" style="margin-top: 5px;">
                          <i class="fa fa-download"></i>
                          <span>XML</span>
                        </a>"""
        $("#extra_buttons").empty().append $(xml_button)

    # $("#extra_buttons").empty().append $(xml_button).click ->
    #     new_window (new XMLSerializer()).serializeToString data
    #     false


    delay_viewport_change()

    address.set_from_xml data

    return
