from selenium import webdriver
from utils import neutral
from tests import Test
import config
import argparse
import sys


# Add command line arguments
parser = argparse.ArgumentParser(description='Test the Sparv interface')
parser.add_argument("-e", "--environ", default="live", type=str, dest='environ',
                    choices=['live', 'lab', 'local'], help='Sparvs testing environment')
args = parser.parse_args()


def main():
    """Main wrapper for Sparv tests."""
    launch_url = config.launch_url[args.environ]
    neutral("\nStarting %s driver..." % config.browser)
    opendriver = Driver(browser=config.browser)
    try:
        Test(opendriver.driver, launch_url)
        neutral("\nDone. Shutting down server.")
        opendriver.teardown()
    except Exception as e:
        neutral("Something went wrong! %s" % e)
        opendriver.teardown()


class Driver:
    """Managing startup and teardown of a web server."""
    def __init__(self, browser=""):
        self.setup(browser)

    def setup(self, browser):
        """Set up the webdriver."""
        assert browser.lower() in ["chrome", "firefox"], "Browser must be firefox or chrome. %s is not valid." % browser
        if browser.lower() == "chrome":
            self.driver = webdriver.Chrome()
        elif browser.lower() == "firefox":
            self.driver = webdriver.Firefox()

    def teardown(self):
        """Shut down webdriver."""
        self.driver.quit()


if __name__ == '__main__':
    main()
