# -*- coding: utf-8 -*-

from utils import passed, failed, neutral, warning
from random import randint
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
import time  # time.sleep(10)


class Test:

    def __init__(self, driver, launch_url):
        self.driver = driver
        self.init_stats()
        self.launch_url = launch_url
        self.run_tests()
        self.print_stats()

    def run_tests(self):
        neutral("\nLaunching on %s\n" % self.launch_url)
        self.driver.get(self.launch_url)

        self.load_advanced_settings()
        self.quick_settings_response()
        self.run_example()

    #######################################
    # THE ACTUAL TESTS
    #######################################

    def load_advanced_settings(self):
        msg = "Advanced settings loaded"
        try:
            # Open advanced settings, check if element "settings_corpus" is present
            self.driver.find_element_by_id("activate_form").click()
            WebDriverWait(self.driver, 3).until(
                EC.presence_of_element_located((By.ID, "settings_corpus")))
            self.pass_test(msg)
        except:
            self.fail_test(msg)

    def quick_settings_response(self):
        msg = "Quick settings response"
        try:
            # Uncheck checkbox compound analysis
            self.driver.find_element_by_id("Dependency_analysis").click()
            # Reset form is clickable
            self.driver.find_element_by_id("reset_form").is_enabled()
            # Dependency attrs are inactive
            for button in self.driver.find_elements_by_css_selector(
                    "#settings_positional_attributes_lexical_attributes button"):
                assert "active" not in button.get_attribute("class")
            self.pass_test(msg)
        except:
            self.fail_test(msg)

    def run_example(self):
        msg = "Running simple example"
        driver = self.driver
        try:
            driver.find_element_by_id("deactivate_form").click()
            # Enter some text and run it, wait for result
            intext = u"Det här är exempelmening %s" % (randint(1, 9) * 987654321)
            driver.execute_script(
                "document.getElementsByClassName('CodeMirror')[0].CodeMirror.setValue('%s')" % intext)
            driver.find_element_by_css_selector("button.btn_submit").click()
            WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.CSS_SELECTOR, "#result > div")))
            driver.execute_script("scroll(0, 500)")
            # TODO Check values in result table
            assert driver.find_element_by_css_selector('#result tr:last-child > td:first-child > span') == "Det"
            self.pass_test(msg)
        except Exception as e:
            print e
            self.fail_test(msg)

    #######################################
    # UTIL METHODS
    #######################################

    def init_stats(self):
        self.failed = 0
        self.passed = 0

    def print_stats(self):
        """Print test statistics."""
        print("\n" + "-" * 40)
        neutral("\tTest statistics:")
        n_tests = self.passed + self.failed
        passed_msg = "Tests passed: %s / %s" % (self.passed, n_tests)
        failed_msg = "Tests failed: %s / %s" % (self.failed, n_tests)
        if self.passed > 0:
            passed(passed_msg)
        else:
            failed("No tests passed!")
        if self.failed == 0:
            passed("No tests failed!")
        else:
            failed(failed_msg)
        print("-" * 40)

    def pass_test(self, msg):
        passed(msg)
        self.passed += 1

    def fail_test(self, msg):
        failed(msg)
        self.failed += 1
